@@ -1,3 +1,4 @@
+
 /*
  * Copyright © 2002 Havoc Pennington
  * Copyright © 2002 Mathias Hasselmann
@@ -210,14 +211,13 @@ static gboolean
 rgba_equal (const GdkRGBA *a,
             const GdkRGBA *b)
 {
-  gdouble dr, dg, db, da;
+  gdouble dr, dg, db;
 
   dr = a->red - b->red;
   dg = a->green - b->green;
   db = a->blue - b->blue;
-  da = a->alpha - b->alpha;
 
-  return (dr * dr + dg * dg + db * db + da * da) < 1e-4;
+  return (dr * dr + dg * dg + db * db) < 1e-4;
 }
 
 static gboolean
@@ -653,6 +653,7 @@ s_to_rgba (GValue *value,
   if (!gdk_rgba_parse (&color, s))
     return FALSE;
 
+  color.alpha = 1.0;
   g_value_set_boxed (value, &color);
   return TRUE;
 }
@@ -755,6 +756,8 @@ bool_to_scrollbar_policy (const GValue *value,
   return g_variant_new_string (g_value_get_boolean (value) ? "always" : "never");
 }
 
+#if !GTK_CHECK_VERSION (3, 19, 8)
+
 /* ATTENTION: HACK HACK HACK!
  * GtkColorButton usability is broken. It always pops up the
  * GtkColorChooserDialog with show-editor=FALSE, which brings
@@ -791,7 +794,8 @@ fixup_color_chooser_button (void)
     done = TRUE;
   }
 }
-/* END HACK */
+
+#endif /* GTK+ < 3.19.8 HACK */
 
 /**
  * terminal_profile_edit:
@@ -826,7 +830,9 @@ terminal_profile_edit (GSettings  *profile,
       return;
     }
 
+#if !GTK_CHECK_VERSION (3, 19, 8)
   fixup_color_chooser_button ();
+#endif
 
   profiles_list = terminal_app_get_profiles_list (terminal_app_get ());
 
@@ -879,6 +885,10 @@ terminal_profile_edit (GSettings  *profile,
       g_snprintf (name, sizeof (name), "palette-colorpicker-%u", i + 1);
       w = (GtkWidget *) gtk_builder_get_object  (builder, name);
 
+#if GTK_CHECK_VERSION (3, 19, 8)
+      g_object_set (w, "show-editor", TRUE, NULL);
+#endif
+
       g_object_set_data (G_OBJECT (w), "palette-entry-index", GUINT_TO_POINTER (i));
 
       text = g_strdup_printf (_("Choose Palette Color %u"), i + 1);
@@ -966,12 +976,15 @@ terminal_profile_edit (GSettings  *profile,
                                 vte_erase_binding_get_type, NULL);
   g_settings_bind (profile, TERMINAL_PROFILE_BOLD_COLOR_SAME_AS_FG_KEY,
                    gtk_builder_get_object (builder,
-                                           "bold-color-same-as-fg-checkbox"),
-                   "active", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+                                           "bold-color-checkbutton"),
+                   "active",
+                   G_SETTINGS_BIND_GET |
+                   G_SETTINGS_BIND_INVERT_BOOLEAN |
+                   G_SETTINGS_BIND_SET);
   g_settings_bind (profile, TERMINAL_PROFILE_BOLD_COLOR_SAME_AS_FG_KEY,
                    gtk_builder_get_object (builder,
-                                           "bold-colorpicker-box"),
-                   "sensitive", 
+                                           "bold-colorpicker"),
+                   "sensitive",
                    G_SETTINGS_BIND_GET |
                    G_SETTINGS_BIND_INVERT_BOOLEAN |
                    G_SETTINGS_BIND_NO_SENSITIVITY);
@@ -979,7 +992,71 @@ terminal_profile_edit (GSettings  *profile,
                                 gtk_builder_get_object (builder,
                                                         "bold-colorpicker"),
                                 "rgba",
-                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET,
+                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
+                                (GSettingsBindGetMapping) s_to_rgba,
+                                (GSettingsBindSetMapping) rgba_to_s,
+                                NULL, NULL);
+  g_settings_bind (profile, TERMINAL_PROFILE_CURSOR_COLORS_SET_KEY,
+                   gtk_builder_get_object (builder,
+                                           "cursor-colors-checkbutton"),
+                   "active", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+  g_settings_bind (profile, TERMINAL_PROFILE_CURSOR_COLORS_SET_KEY,
+                   gtk_builder_get_object (builder,
+                                           "cursor-foreground-colorpicker"),
+                   "sensitive",
+                   G_SETTINGS_BIND_GET |
+                   G_SETTINGS_BIND_NO_SENSITIVITY);
+  g_settings_bind (profile, TERMINAL_PROFILE_CURSOR_COLORS_SET_KEY,
+                   gtk_builder_get_object (builder,
+                                           "cursor-background-colorpicker"),
+                   "sensitive",
+                   G_SETTINGS_BIND_GET |
+                   G_SETTINGS_BIND_NO_SENSITIVITY);
+  g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_CURSOR_FOREGROUND_COLOR_KEY,
+                                gtk_builder_get_object (builder,
+                                                        "cursor-foreground-colorpicker"),
+                                "rgba",
+                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
+                                (GSettingsBindGetMapping) s_to_rgba,
+                                (GSettingsBindSetMapping) rgba_to_s,
+                                NULL, NULL);
+  g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_CURSOR_BACKGROUND_COLOR_KEY,
+                                gtk_builder_get_object (builder,
+                                                        "cursor-background-colorpicker"),
+                                "rgba",
+                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
+                                (GSettingsBindGetMapping) s_to_rgba,
+                                (GSettingsBindSetMapping) rgba_to_s,
+                                NULL, NULL);
+  g_settings_bind (profile, TERMINAL_PROFILE_HIGHLIGHT_COLORS_SET_KEY,
+                   gtk_builder_get_object (builder,
+                                           "highlight-colors-checkbutton"),
+                   "active", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+  g_settings_bind (profile, TERMINAL_PROFILE_HIGHLIGHT_COLORS_SET_KEY,
+                   gtk_builder_get_object (builder,
+                                           "highlight-foreground-colorpicker"),
+                   "sensitive",
+                   G_SETTINGS_BIND_GET |
+                   G_SETTINGS_BIND_NO_SENSITIVITY);
+  g_settings_bind (profile, TERMINAL_PROFILE_HIGHLIGHT_COLORS_SET_KEY,
+                   gtk_builder_get_object (builder,
+                                           "highlight-background-colorpicker"),
+                   "sensitive",
+                   G_SETTINGS_BIND_GET |
+                   G_SETTINGS_BIND_NO_SENSITIVITY);
+  g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_HIGHLIGHT_FOREGROUND_COLOR_KEY,
+                                gtk_builder_get_object (builder,
+                                                        "highlight-foreground-colorpicker"),
+                                "rgba",
+                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
+                                (GSettingsBindGetMapping) s_to_rgba,
+                                (GSettingsBindSetMapping) rgba_to_s,
+                                NULL, NULL);
+  g_settings_bind_with_mapping (profile, TERMINAL_PROFILE_HIGHLIGHT_BACKGROUND_COLOR_KEY,
+                                gtk_builder_get_object (builder,
+                                                        "highlight-background-colorpicker"),
+                                "rgba",
+                                G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET | G_SETTINGS_BIND_NO_SENSITIVITY,
                                 (GSettingsBindGetMapping) s_to_rgba,
                                 (GSettingsBindSetMapping) rgba_to_s,
                                 NULL, NULL);
